<?php
/*               *** sqstat - Squid Proxy Server realtime stat ***
                        (c) Alex Samorukov, samm@os2.kiev.ua
*/

// Squid proxy server ip address or host name
$squidhost="127.0.0.1";
// Squid proxy server port
$squidport=3128;
// cachemgr_passwd in squid.conf. Leave blank to disable authorisation
$cachemgr_passwd="";
// Resolve user IP addresses or print them as numbers only [true|false]
$resolveip=false; 
// Maximum URL length to display in URI table column
DEFINE("SQSTAT_SHOWLEN",80);

/* Program body */
DEFINE("SQSTAT_VERSION","0.95");
$squidclass=new squidstat();
if(!$squidclass->connect($squidhost,$squidport)) {
	$squidclass->showError();
	exit(1);
}
$data=$squidclass->makeQuery($cachemgr_passwd);
if($data==false){
	$squidclass->showError();
	exit(2);
}
$squidclass->makeHtmlReport($data,$resolveip);

class squidstat{
	var $fp;
	var $errstr;
	var $errno;
	
	
	function formatXHTML($body,$refresh){
		$text='<?xml version="1.0" encoding="UTF-8"?>'."\n".
		'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'."\n"
		.'<html>'
		.'<head>'
		.'<link href="sqstat.css" rel="stylesheet" type="text/css"/>';
		if($refresh) $text.='<META HTTP-EQUIV=Refresh CONTENT="'.$refresh.'; URL='.$_SERVER["PHP_SELF"].'?refresh='.$refresh.'"/>';
		$text.='<title>SqStat '.SQSTAT_VERSION.'</title>'
		.'</head><body>'
		.$body
		.'</body></html>';
		return $text;
	}
	
	function showError(){
		$text='<h1>SqStat error</h1>'.
		'<h2 style="color:red">Error ('.$this->errno.'): '.$this->errstr.'</span>';
		echo $this->formatXHTML($text,0);
	}
	function connect($squidhost,$squidport){
		$this->fp = false;
		// Connecting to the squidhost
		$this->fp = @fsockopen($squidhost, $squidport, $this->errno, $this->errstr, 10);
		if (!$this->fp) {
			// failed to connect
			return false;
		}
		return true;
	}
	
	// based @ (c) moritz at barafranca dot com
	function duration ($seconds) {
		$takes_time = array(604800,86400,3600,60,0);
		$suffixes = array("w","d","h","m","s");
		$output = "";
		foreach ($takes_time as $key=>$val) {
			${$suffixes[$key]} = ($val == 0) ? $seconds : floor(($seconds/$val));
			$seconds -= ${$suffixes[$key]} * $val;
			if (${$suffixes[$key]} > 0) {
				$output .=  ${$suffixes[$key]};
				$output .= $suffixes[$key]." ";
			}
		}
		return trim($output);
	}
	/**
	* Format a number of bytes into a human readable format.
	* Optionally choose the output format and/or force a particular unit
	*
	* @param   int     $bytes      The number of bytes to format. Must be positive
	* @param   string  $format     Optional. The output format for the string
	* @param   string  $force      Optional. Force a certain unit. B|KB|MB|GB|TB
	* @return  string              The formatted file size
	*/
    function filesize_format($bytes, $format = '', $force = '')
    {
        $force = strtoupper($force);
        $defaultFormat = '%01d %s';
        if (strlen($format) == 0)
		$format = $defaultFormat;
        $bytes = max(0, (int) $bytes);
        $units = array('b', 'Kb', 'Mb', 'Gb', 'Tb', 'Pb');
        $power = array_search($force, $units);
        if ($power === false)
		$power = $bytes > 0 ? floor(log($bytes, 1024)) : 0;
        return sprintf($format, $bytes / pow(1024, $power), $units[$power]);
    }
	function makeQuery($pass=""){
		$raw=array();
		// sending request
		if(!$this->fp) die("Please connect to server");
		$out = "GET cache_object://localhost/active_requests HTTP/1.0\r\n";
		if($pass!="") $out.="Authorization: Basic ".base64_encode("cachemgr:$pass")."\r\n";
		$out.="\r\n";
		fwrite($this->fp, $out);
		
		while (!feof($this->fp)) {
			$raw[]=trim(fgets($this->fp, 2048));
		}
		fclose($this->fp);
		
		if($raw[0]!="HTTP/1.0 200 OK"){
			$this->errno=1;
			$this->errstr="Cannot get data. Server answered: $raw[0]";
			return false;
		}
		
		// parsing output;
		
		$header=1;
		$connection=0;
		foreach($raw as $key=>$v){
			// cutoff http header
			if($header==1 && $v=="") $header=0;
			if($header){
				if(substr($v,0,7)=="Server:"){ // parsing server version
					$parsed["server_version"]=substr($v,8);
				}
			}
			else {
				if(substr($v,0,11)=="Connection:"){ // parsing connection
					$connection=substr($v,12);
				}
				if($connection){
					if(substr($v,0,5)=="peer:") $parsed["con"][$connection]["peer"]=substr($v,6);
					if(substr($v,0,4)=="uri ") $parsed["con"][$connection]["uri"]=substr($v,4);
					if(preg_match('/out.offset \d+, out.size (\d+)/',$v,$matches)){
						$parsed["con"][$connection]["bytes"]=$matches[1];
					}
					if(preg_match('/start \d+\.\d+ \((\d+).\d+ seconds ago\)/',$v,$matches)){
						$parsed["con"][$connection]["seconds"]=$matches[1];
					}
				}
			}
		}
		return $parsed;
	}
	function makeHtmlReport($data,$resolveip=false){
		// resort data array
		$users=array();
		foreach($data["con"] as $v){
			if(substr($v["uri"],0,13)=="cache_object:") continue; // skip myself
			// i user ip2long() to make ip sorting work correctly
			if($resolveip){
				$ip=substr($v["peer"],0,strpos($v["peer"],":"));
				$hostname=gethostbyaddr($ip);
				if($hostname==$ip) $ip=ip2long($ip);// resolve failed
				else $ip=$hostname;
			}
			else{
				$ip=ip2long(substr($v["peer"],0,strpos($v["peer"],":")));
			}
			$users[$ip][]=$v;
		}
		ksort($users);
		$refresh=0;
		if(isset($_GET["refresh"]) && !isset($_GET["stop"])) $refresh=(int)$_GET["refresh"];
		$text='';
		$text.='<div class="header"><form method="get" action="'.$_SERVER["PHP_SELF"].'">'.
		'Squid RealTime stat for the '.$GLOBALS["squidhost"].':'.$GLOBALS["squidport"].' proxy server ('.$data["server_version"].').<br/>'.
		'Auto refresh: <input name="refresh" type="text" size="4" value="'.$refresh.'"/> sec. <input type="submit" value="Update"/> <input name="stop" type="submit" value="Stop"/> Created at: <tt>'.date("h:i:s d/m/Y").'</tt><br/>'.
		'</div>'.
		'<table class="result" align="center" width="100%" border="0">'.
		'<tr>'.
		'<th>User</th><th>URI</th><th>Size</th><th>Time</th>'.
		'</tr>';
		$ausers=$acon=0;
		foreach($users as $key=>$v){
			$ausers++;
			$text.='<tr><td  style="border-right:0;"><b>'.(is_int($key)?long2ip($key):$key).'</b></td><td style="border-left:0;" colspan="3">&nbsp;</td></tr>';
			foreach ($v as $con){
				if(substr($con["uri"],0,7)=="http://" || substr($con["uri"],0,6)=="ftp://"){
					if(strlen($con["uri"])>SQSTAT_SHOWLEN) $uritext=substr($con["uri"],0,SQSTAT_SHOWLEN).'</a> ....';
					else $uritext=$con["uri"].'</a>';
					$uri='<a target="_blank" href="'.$con["uri"].'" title="'.$con["uri"].'">'.$uritext;
				}
				else $uri=$con["uri"];
				$acon++;
				$text.='<tr><td></td>'.
				'<td nowrap width="80%">'.$uri.'</td>'.
				'<td nowrap align="right">'.$this->filesize_format($con["bytes"]).'</td>'.
				'<td nowrap align="right">'.$this->duration($con["seconds"],"short").'</td>'.
				'</tr>';
			}
		}
		$text.='<tr class="total"><td><b>Total:</b></td><td align="right" colspan="3"> <b>'.$ausers.'</b> users and <b>'.$acon.'</b> connections</td></tr>'.
		'</table>'.
		'<p class="copyleft">&copy; <a href="mailto:samm@os2.kiev.ua">Alex Samorukov</a>, 2006</p>';
		echo $this->formatXHTML($text,$refresh);
	}
}
?>